pub fn param_binding_element_obj_for_store(
name: String,
binding_kind: ParamObjType,
) -> ObjExpand description
Element Obj for stored typing / membership facts so keys match parsed bound names (~tag spine).
pub fn param_binding_element_obj_for_store(
name: String,
binding_kind: ParamObjType,
) -> ObjElement Obj for stored typing / membership facts so keys match parsed bound names (~tag spine).